@@ -35,14 +35,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 
-# TODO: Consider setting up so if no input file argument and the gui is used then
+# Todo: Consider setting up so if no input file argument and the gui is used then
 # the file chooser will pop up.
 
-# TODO: Maybe use spinner for orderStat, checkbox for uniform and samepagesize.
-# Maybe not.
+# Todo: Maybe use a spinner for the orderStat entries.
 
-# TODO: Look into the new Sizer in pySimpleGUI to see if the size of the PDF
-# window can be fixed to the initial size or something similar.
+# Todo: Look into the new Sizer in pySimpleGUI to see if the size of the PDF
+# window can (or should) be fixed to the initial size or something similar.
 
 # Note: When using gs for bboxes the threshold is not allowed (or blurs or
 # smooths).  Currently writing "---" but would look better disabled altogether
@@ -50,23 +49,19 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # pysimplegui issues: 1) warning on Window title in Python2 on some machines, 2) tooltips
 # need pointer to move left to work, 3) non-string initial values not set for Combo.
-# The showing/hiding of the `Calculating the crop...` message doesn't work on Windows.
 
 from __future__ import print_function, absolute_import
 
 import sys
 import os
 import warnings
+import textwrap
+
 from . import __version__
 from . import external_program_calls as ex
+from . pymupdf_routines import open_document, get_page
 
-try:
-    with warnings.catch_warnings():
-        #warnings.filterwarnings("ignore",category=DeprecationWarning)
-        requires = "PyMuPDF at least v1.14.5"
-        import fitz
-    if not [int(i) for i in fitz.VersionBind.split(".")] >= [1, 14, 5]:
-        raise ImportError
+try: # Extra dependencies for the GUI version.  Make sure they are installed.
     if not ex.python_version[0] == "2":
         requires = "PySimpleGUI"
         import PySimpleGUI as sg
@@ -87,9 +82,10 @@ except ImportError:
 from .main_pdfCropMargins import (process_pdf_file, parse_page_range_specifiers,
                                   parse_page_ratio_argument)
 
-# Look and feel.
+# Uncomment for look and feel preview.
 #print(sg.ListOfLookAndFeelValues())
 #print(sg.LOOK_AND_FEEL_TABLE)
+
 if ex.system_os == "Windows":
     sg.ChangeLookAndFeel("TanBlue")
 else:
@@ -101,6 +97,8 @@ else:
 
 def get_filename():
     """Get the filename of the PDF file via GUI if one was not passed in."""
+    # TODO: This isn't used now, but the code works to pop up a file chooser.
+    # Incorporate it to interactively find the file to crop if none passed in.
     fname = sg.PopupGetFile("Select file and filetype to open:",
                             title="pdfCropMargins: Document Browser",
                             file_types=[ # Only PDF files.
@@ -109,72 +107,6 @@ def get_filename():
                             )
     return fname # Might be None.
 
-def open_document(doc_fname):
-    """Return the document opened by fitz (PyMuPDF)."""
-    # TODO: Move the get_filename call to main_pdfCropMargins or
-    # similar if actually used.
-    if not doc_fname:
-        doc_fname = get_filename()
-    try:
-        document = fitz.open(doc_fname)
-    except RuntimeError:
-        print("\nError in pdfCropMargins: The PyMuPDF program could not read"
-              " the document\n   '{}'\nin order to display it in the GUI.   If you have"
-              " Ghostscript installed\nconsider running pdfCropMargins with the"
-              " '--gsFix' option to attempt to repair it."
-              .format(doc_fname), file=sys.stderr)
-        ex.cleanup_and_exit(1)
-    page_count = len(document)
-    return document, page_count
-
-def get_page(page_num, page_display_list_cache, document, window_size, zoom=False):
-    """Return a `tkinter.PhotoImage` or a PNG image for a document page number.
-    - The `page_num` argument is a 0-based page number.
-    - The `zoom` argument is the top-left of old clip rect, and one of -1, 0,
-      +1 for dim. x or y to indicate the arrow key pressed.
-    - The `max_size` argument is the (width, height) of available image area.
-      """
-    zoom_x = 1
-    zoom_y = 1
-    scale = fitz.Matrix(zoom_x, zoom_y)
-    page_display_list = page_display_list_cache[page_num]
-    if not page_display_list:  # Create if not yet there.
-        page_display_list_cache[page_num] = document[page_num].getDisplayList()
-        page_display_list = page_display_list_cache[page_num]
-
-    rect = page_display_list.rect  # The page rectangle.
-    clip = rect
-
-    # Make sure that the image will fits the screen.
-    zoom_0 = 1
-    if window_size:
-        zoom_0 = min(1, window_size[0] / rect.width, window_size[1] / rect.height)
-        if zoom_0 == 1:
-            zoom_0 = min(window_size[0] / rect.width, window_size[1] / rect.height)
-    mat_0 = fitz.Matrix(zoom_0, zoom_0)
-
-    if not zoom:  # Show the total page.
-        pixmap = page_display_list.getPixmap(matrix=mat_0, alpha=False)
-    else:
-        w2 = rect.width / 2  # we need these ...
-        h2 = rect.height / 2  # a few times
-        clip = rect * 0.5  # clip rect size is a quarter page
-        tl = zoom[0]  # old top-left
-        tl.x += zoom[1] * (w2 / 2)  # adjust top-left ...
-        tl.x = max(0, tl.x)  # according to ...
-        tl.x = min(w2, tl.x)  # arrow key ...
-        tl.y += zoom[2] * (h2 / 2)  # provided, but ...
-        tl.y = max(0, tl.y)  # stay within ...
-        tl.y = min(h2, tl.y)  # the page rect
-        clip = fitz.Rect(tl, tl.x + w2, tl.y + h2)
-
-        # Clip rect is ready, now fill it.
-        mat = mat_0 * fitz.Matrix(2, 2)  # The zoom matrix.
-        pixmap = page_display_list.getPixmap(alpha=False, matrix=mat, clip=clip)
-
-    image_ppm = pixmap.getImageData("ppm")  # Make PPM image from pixmap for tkinter.
-    return image_ppm, clip.tl  # Return image, clip position.
-
 def get_window_size():
     """Get physical screen dimension to determine the page image max size."""
     root = tk.Tk()
@@ -183,11 +115,12 @@ def get_window_size():
     root.destroy()
     return width, height
 
+wrapper = textwrap.TextWrapper(initial_indent="", subsequent_indent="", width=45,
+                               break_on_hyphens=False)
+
 def get_help_text_string_for_tooltip(cmd_parser, option_string):
     """Extract the help message for an option from an argparse command parser.
     This gets the argparse help string to use as a tooltip."""
-    import textwrap
-    wrapper = textwrap.TextWrapper(initial_indent="", subsequent_indent="", width=45)
     for a in cmd_parser._actions:
         if "--" + option_string in a.option_strings:
             help_text = a.help
@@ -256,7 +189,18 @@ def update_combo_box(values_dict, element, element_key, args, attr, fun_to_apply
         return
     element_value = values_dict[element_key]
     element_value = fun_to_apply(element_value)
-    element.Update(str(element_value))
+    #element.Update(str(element_value)) # Redundant.
+    setattr(args, attr, element_value)
+
+def update_checkbox(values_dict, element, element_key, args, attr, fun_to_apply=None):
+    """Update a non-paired, independent option like `uniform`.  Function `fun_to_apply`
+    is applied to the GUI value to convert it to the type `args` expects."""
+    if values_dict is None:
+        return
+    element_value = values_dict[element_key]
+    if fun_to_apply:
+        element_value = fun_to_apply(element_value)
+    #element.Update(element_value) # Redundant.
     setattr(args, attr, element_value)
 
 def update_4_values(element_list, attr, args_dict, values_dict, value_type=float):
@@ -321,6 +265,53 @@ def update_paired_1_and_4_values(element, element_list4, attr, attr4, args_dict,
     # Update all, to convert forms like 5 to 5.0 (which were equal above).
     update_all_from_args_dict()
 
+##
+## Define the buttons/events we want to handle in the event loop.
+##
+
+class Events:
+    """The events to handle in the event loop.  The class is just used as a
+    namespace for holding the event tests."""
+    # When no longer supporting Python2 consider making this a SimpleNamespace instance.
+    def is_enter(btn):
+        return btn.startswith("Return:") or btn == chr(13)
+
+    def is_exit(btn):
+        return btn == chr(27) or btn.startswith("Escape:") or btn.startswith("Exit")
+
+    def is_crop(btn):
+        return btn.startswith("Crop")
+
+    def is_original(btn):
+        return btn.startswith("Original")
+
+    def is_next(btn):
+        return btn.startswith("Next") or btn == "MouseWheel:Down" # Note mouse not giving any event.
+
+    def is_prev(btn):
+        return btn.startswith("Prior:") or btn.startswith("Prev") or btn == "MouseWheel:Up"
+
+    def is_up(btn):
+        return btn.startswith("Up:")
+
+    def is_down(btn):
+        return btn.startswith("Down:")
+
+    def is_home(btn):
+        return btn.startswith("Home:")
+
+    def is_end(btn):
+        return btn.startswith("End:")
+
+    def is_left(btn):
+        return btn.startswith("Left:")
+
+    def is_right(btn):
+        return btn.startswith("Right:")
+
+    def is_zoom(btn):
+        return btn.startswith("Toggle Zoom")
+
 #
 # The main function with the event loop.
 #
@@ -336,8 +327,8 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     ## Set up the document and window.
     ##
 
-    document, page_count = open_document(fixed_input_doc_fname)
-    cur_page = 0
+    document, num_pages = open_document(fixed_input_doc_fname)
+    curr_page = 0
 
     window_title = "pdfCropMargins: {}".format(os.path.basename(input_doc_fname))
     window_size = get_window_size()
@@ -347,14 +338,13 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
         warnings.filterwarnings("ignore", message="Your title is not a string.")
         window = sg.Window(title=window_title, return_keyboard_events=True, location=(0, 0),
                            use_default_focus=False)
-    # On tooltips done by pySimpleGUI in tkinter, it seems you have to move pointer
-    # left over the text to make them appear...
+
     sg.SetOptions(tooltip_time=500)
 
     # Allocate storage for caching page display lists.
-    page_display_list_cache = [None] * page_count
+    page_display_list_cache = [None] * num_pages
 
-    data, clip_pos = get_page(cur_page,  # Read first page.
+    data, clip_pos = get_page(curr_page,  # Read first page.
                               page_display_list_cache,
                               document,
                               window_size=size_for_full_app,  # image max dim
@@ -369,18 +359,18 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     ## Code for handling page numbers.
     ##
 
-    input_text_page_num = sg.InputText(str(cur_page + 1), size=(5, 1),
+    input_text_page_num = sg.InputText(str(curr_page + 1), size=(5, 1),
                                        do_not_clear=True, key="PageNumber")
     text_page_num = sg.Text("Page:")
 
-    def update_page_number(cur_page, page_count, is_page_mod_key, btn,
+    def update_page_number(curr_page, prev_curr_page, num_pages, btn,
                            input_text_element):
-        cur_page = max(cur_page, 0)
-        cur_page = min(cur_page, page_count-1)
+        curr_page = max(curr_page, 0)
+        curr_page = min(curr_page, num_pages-1)
         # Update page number field.
-        if is_page_mod_key(btn):
-            input_text_element.Update(str(cur_page + 1))
-        return cur_page
+        if curr_page != prev_curr_page:
+            input_text_element.Update(str(curr_page + 1))
+        return curr_page
 
     ##
     ## Code for percentRetain options.
@@ -435,7 +425,8 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     def update_absoluteOffset_values(values_dict):
         """Update both the absoluteOffset value and the absoluteOffset4 values."""
         update_paired_1_and_4_values(input_text_absoluteOffset,
-                    input_text_absoluteOffset4, "absoluteOffset", "absoluteOffset4", args_dict, values_dict)
+                    input_text_absoluteOffset4, "absoluteOffset", "absoluteOffset4",
+                    args_dict, values_dict)
         # Copy backing values to the actual args object.
         args.absoluteOffset = args_dict["absoluteOffset"]
         args.absoluteOffset4 = args_dict["absoluteOffset4"]
@@ -450,6 +441,7 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
         args_dict["uniformOrderStat"] = args.uniformOrderStat
     else:
         args_dict["uniformOrderStat"] = [0]
+
     if args.uniformOrderStat4:
         args_dict["uniformOrderStat4"] = args.uniformOrderStat4
         if len(set(args.uniformOrderStat4)) != 1: # Set initial value if all the same.
@@ -460,6 +452,7 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
         args_dict["uniformOrderStat4"] = [args.uniformOrderStat[0]] * 4
     else:
         args_dict["uniformOrderStat4"] = [0] * 4
+
     text_uniformOrderStat = sg.Text("uniformOrderStat",
                       tooltip=get_help_text_string_for_tooltip(cmd_parser, "uniformOrderStat"))
     input_text_uniformOrderStat = sg.InputText(args_dict["uniformOrderStat"][0],
@@ -484,6 +477,12 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
             args.uniformOrderStat4 = [] # Need to empty it, since it implies uniform option.
         else:
             args.uniformOrderStat4 = args_dict["uniformOrderStat4"]
+        # Disable the uniform checkbox (this option implies uniform cropping).
+        if args.uniformOrderStat4:
+            checkbox_uniform.Update(disabled=True)
+        else:
+            checkbox_uniform.Update(disabled=False)
+
 
     update_funs.append(update_uniformOrderStat_values)
 
@@ -491,17 +490,14 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     ## Code for uniform.
     ##
 
-    text_uniform = sg.Text("uniform", pad=((0,20), None),
-                      tooltip=get_help_text_string_for_tooltip(cmd_parser, "uniform"))
-
-    # BUG in PySimpleGUI?  Default values not set unless they are strings!
-    combo_box_uniform = sg.Combo(["True", "False"], readonly=True, default_value=str(args.uniform),
-                                       size=(5, 1), key="uniform", enable_events=True)
+    checkbox_uniform = sg.Checkbox("uniform", pad=((0,10), None), key="uniform",
+                                    tooltip=get_help_text_string_for_tooltip(cmd_parser,
+                                        "uniform"),
+                                    enable_events=True, default=args.uniform)
 
     def update_uniform(values_dict):
         """Update the uniform values."""
-        update_combo_box(values_dict, combo_box_uniform, "uniform", args, "uniform",
-                        fun_to_apply=str_to_bool)
+        update_checkbox(values_dict, checkbox_uniform, "uniform", args, "uniform")
 
     update_funs.append(update_uniform)
 
@@ -509,20 +505,35 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     ## Code for samePageSize.
     ##
 
-    text_samePageSize = sg.Text("samePageSize",
-                      tooltip=get_help_text_string_for_tooltip(cmd_parser, "samePageSize"))
-
-    combo_box_samePageSize = sg.Combo(["True", "False"], readonly=True,
-                                         default_value=str(args.samePageSize), size=(5, 1),
-                                         key="samePageSize", enable_events=True)
+    checkbox_samePageSize = sg.Checkbox("samePageSize", pad=((0,10), None),
+                                         key="samePageSize", enable_events=True,
+                                         tooltip=get_help_text_string_for_tooltip(
+                                             cmd_parser, "samePageSize"),
+                                         default=args.samePageSize)
 
     def update_samePageSize(values_dict):
         """Update the samePageSize values."""
-        update_combo_box(values_dict, combo_box_samePageSize, "samePageSize", args, "samePageSize",
-                        fun_to_apply=str_to_bool)
+        update_checkbox(values_dict, checkbox_samePageSize, "samePageSize", args,
+                         "samePageSize")
 
     update_funs.append(update_samePageSize)
 
+    ##
+    ## Code for evenodd option.
+    ##
+
+    checkbox_evenodd = sg.Checkbox("evenodd", pad=((0,0), None),
+                                    key="evenodd", enable_events=True,
+                                    tooltip=get_help_text_string_for_tooltip(
+                                        cmd_parser, "evenodd"),
+                                    default=args.evenodd)
+
+    def update_evenodd(values_dict):
+        """Update the evenodd values."""
+        update_checkbox(values_dict, checkbox_evenodd, "evenodd", args, "evenodd")
+
+    update_funs.append(update_evenodd)
+
     ##
     ## Code for pages option.
     ##
@@ -538,7 +549,7 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
         value = values_dict["pages"]
         try:
             if value:
-                parse_page_range_specifiers(value, set(range(page_count)))
+                parse_page_range_specifiers(value, set(range(num_pages)))
         except ValueError:
             sg.PopupError("Bad page specifier.")
             input_text_pages.Update("")
@@ -551,22 +562,22 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     update_funs.append(update_pages_values)
 
     ##
-    ## Code for evenodd option.
+    ## Code for restore.
     ##
 
-    text_evenodd = sg.Text("evenodd",
-                      tooltip=get_help_text_string_for_tooltip(cmd_parser, "evenodd"))
+    text_restore = sg.Text("restore",
+                      tooltip=get_help_text_string_for_tooltip(cmd_parser, "restore"))
 
-    # BUG: default values not set unless they are strings!
-    combo_box_evenodd = sg.Combo(["True", "False"], readonly=True, default_value=str(args.evenodd),
-                                       size=(5, 1), key="evenodd", enable_events=True)
+    combo_box_restore = sg.Combo(["True", "False"], readonly=True,
+                                         default_value=str(args.restore), size=(5, 1),
+                                         key="restore", enable_events=True)
 
-    def update_evenodd(values_dict):
-        """Update the evenodd values."""
-        update_combo_box(values_dict, combo_box_evenodd, "evenodd", args, "evenodd",
+    def update_restore(values_dict):
+        """Update the restore values."""
+        update_combo_box(values_dict, combo_box_restore, "restore", args, "restore",
                         fun_to_apply=str_to_bool)
 
-    update_funs.append(update_evenodd)
+    update_funs.append(update_restore)
 
     ##
     ## Code for setPageRatios option.
@@ -598,24 +609,6 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
 
     update_funs.append(update_setPageRatios_values)
 
-    ##
-    ## Code for restore.
-    ##
-
-    text_restore = sg.Text("restore",
-                      tooltip=get_help_text_string_for_tooltip(cmd_parser, "restore"))
-
-    combo_box_restore = sg.Combo(["True", "False"], readonly=True,
-                                         default_value=str(args.restore), size=(5, 1),
-                                         key="restore", enable_events=True)
-
-    def update_restore(values_dict):
-        """Update the restore values."""
-        update_combo_box(values_dict, combo_box_restore, "restore", args, "restore",
-                        fun_to_apply=str_to_bool)
-
-    update_funs.append(update_restore)
-
     ##
     ## Code for pageRatioWeights options.
     ##
@@ -659,7 +652,8 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     def update_absolutePreCrop_values(values_dict):
         """Update both the absolutePreCrop value and the absolutePreCrop4 values."""
         update_paired_1_and_4_values(input_text_absolutePreCrop,
-                    input_text_absolutePreCrop4, "absolutePreCrop", "absolutePreCrop4", args_dict, values_dict)
+                    input_text_absolutePreCrop4, "absolutePreCrop", "absolutePreCrop4",
+                    args_dict, values_dict)
         # Copy backing values to the actual args object.
         args.absolutePreCrop = args_dict["absolutePreCrop"]
         args.absolutePreCrop4 = args_dict["absolutePreCrop4"]
@@ -771,7 +765,7 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
             sg.Button("Next"),
             text_page_num,
             input_text_page_num,
-            sg.Text("({})      ".format(page_count)), # Show max page count.
+            sg.Text("({})      ".format(num_pages)), # Show max page count.
             sg.Button("Toggle Zoom"),
             sg.Text("(arrow keys navigate while zooming)"),
             ],
@@ -781,7 +775,7 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
                     [sg.Text("Quadruples are left, bottom, right, and top margins.\n"
                              "Move mouse left over option names to show descriptions.",
                              relief=sg.RELIEF_GROOVE, pad=(None, (0,15)))],
-                    [combo_box_uniform, text_uniform, combo_box_samePageSize, text_samePageSize],
+                    [checkbox_uniform, checkbox_samePageSize, checkbox_evenodd],
                     [input_text_percentRetain, text_percentRetain],
                     # Python 2 can't unpack in list, so use comprehension.
                     [i for i in input_text_percentRetain4] + [text_percentRetain4],
@@ -789,13 +783,13 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
                     [i for i in input_text_absoluteOffset4] + [text_absoluteOffset4],
                     [input_text_uniformOrderStat, text_uniformOrderStat],
                     [i for i in input_text_uniformOrderStat4] + [text_uniformOrderStat4],
-                    [input_text_pages, text_pages, combo_box_evenodd, text_evenodd],
-                    [input_text_setPageRatios, text_setPageRatios, combo_box_restore, text_restore],
+                    [input_text_setPageRatios, text_setPageRatios],
                     [i for i in input_text_pageRatioWeights] + [text_pageRatioWeights],
                     [input_text_absolutePreCrop, text_absolutePreCrop],
                     [i for i in input_text_absolutePreCrop4] + [text_absolutePreCrop4],
                     [input_num_threshold, text_threshold, input_num_numBlurs,
                         text_numBlurs, input_num_numSmooths, text_numSmooths],
+                    [input_text_pages, text_pages, combo_box_restore, text_restore],
                     [sg.Button("Crop"), sg.Button("Original"), sg.Button("Exit"),],
                     [sg.Text("")], # This is just for vertical space.
                     [sg.Text("", size=(5, 2)), wait_indicator_text],
@@ -804,61 +798,9 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
         ]
 
     window.Layout(layout)
-
-    # Note from manual: "If you want to call an element's Update method or call
-    # a Graph element's drawing primitives, you must either call Read or
-    # Finalize prior to making those calls."
-    #btn, values_dict = window.Read(timeout=0) # Now use Finalize below.
-    #call_all_update_funs(update_funs, values_dict)
-
-    # For some reason I also had to add this in order to make the `wait_indicator_text`
-    # properly become visible and invisible.  Fixes a problem that occurred with newer
-    # PySimpleGU versions in 2020.  The box is initialized above as visible, but
-    # here it is updated to be invisible (until a crop is being performed).  Otherwise
-    # it never displays at all.
-    window.Finalize()
+    window.Finalize() # Newer pySimpleGui versions have finalize kwarg in window def.
     wait_indicator_text.Update(visible=False)
 
-    ##
-    ## Define the buttons/events we want to handle.
-    ##
-
-    def is_enter(btn):
-        return btn.startswith("Return:") or btn == chr(13)
-
-    def is_exit(btn):
-        return btn == chr(27) or btn.startswith("Escape:") or btn.startswith("Exit")
-
-    def is_crop(btn):
-        return btn.startswith("Crop")
-
-    def is_original(btn):
-        return btn.startswith("Original")
-
-    def is_next(btn):
-        return btn.startswith("Next") or btn == "MouseWheel:Down" # Note mouse not giving any event.
-
-    def is_prev(btn):
-        return btn.startswith("Prior:") or btn.startswith("Prev") or btn == "MouseWheel:Up"
-
-    def is_up(btn):
-        return btn.startswith("Up:")
-
-    def is_down(btn):
-        return btn.startswith("Down:")
-
-    def is_left(btn):
-        return btn.startswith("Left:")
-
-    def is_right(btn):
-        return btn.startswith("Right:")
-
-    def is_zoom(btn):
-        return btn.startswith("Toggle Zoom")
-
-    def is_page_mod_key(btn):
-        return any((is_enter(btn), is_next(btn), is_prev(btn), is_zoom(btn)))
-
     ##
     ## Run the main event loop.
     ##
@@ -873,45 +815,52 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
     last_numBlurs = None
 
     while True:
+        prev_curr_page = curr_page
         btn, values_dict = window.Read()
 
         if btn is None and (values_dict is None or values_dict["PageNumber"] is None):
             break
-        if is_exit(btn):
+        if Events.is_exit(btn):
             break
 
-        if is_enter(btn):
+        if Events.is_enter(btn):
             call_all_update_funs(update_funs, values_dict)
             try:
-                cur_page = int(values_dict["PageNumber"]) - 1  # check if valid
+                curr_page = int(values_dict["PageNumber"]) - 1  # check if valid
             except:
-                cur_page = 0
+                curr_page = 0
 
-        elif is_next(btn):
-            cur_page += 1
+        elif Events.is_next(btn):
+            curr_page += 1
 
-        elif is_prev(btn):
-            cur_page -= 1
+        elif Events.is_prev(btn):
+            curr_page -= 1
 
-        elif is_up(btn) and zoom:
+        elif Events.is_up(btn) and zoom:
             zoom = (clip_pos, 0, -1)
 
-        elif is_down(btn) and zoom:
+        elif Events.is_down(btn) and zoom:
             zoom = (clip_pos, 0, 1)
 
-        elif is_left(btn) and zoom:
+        elif Events.is_home(btn):
+            curr_page = 0
+
+        elif Events.is_end(btn):
+            curr_page = num_pages - 1
+
+        elif Events.is_left(btn) and zoom:
             zoom = (clip_pos, -1, 0)
 
-        elif is_right(btn) and zoom:
+        elif Events.is_right(btn) and zoom:
             zoom = (clip_pos, 1, 0)
 
-        elif is_zoom(btn): # Toggle.
+        elif Events.is_zoom(btn): # Toggle.
             if not zoom:
                 zoom = (clip_pos, 0, 0)
             else:
                 zoom = False
 
-        elif is_crop(btn):
+        elif Events.is_crop(btn):
             call_all_update_funs(update_funs, values_dict)
             document.close()
 
@@ -959,27 +908,27 @@ def create_gui(input_doc_fname, fixed_input_doc_fname, output_doc_fname,
                 combo_box_restore.Update("False")
 
             # Change the view to the new cropped file.
-            page_display_list_cache = [None] * page_count
-            document, page_count = open_document(output_doc_fname)
+            page_display_list_cache = [None] * num_pages
+            document, num_pages = open_document(output_doc_fname)
             did_crop = True
             wait_indicator_text.Update(visible=False)
 
             if parsed_args.verbose:
                 print("\nWaiting for the GUI...")
 
-        elif is_original(btn):
+        elif Events.is_original(btn):
             call_all_update_funs(update_funs, values_dict)
             document.close()
-            page_display_list_cache = [None] * page_count
-            document, page_count = open_document(fixed_input_doc_fname)
+            page_display_list_cache = [None] * num_pages
+            document, num_pages = open_document(fixed_input_doc_fname)
             did_crop = False
 
         # Update page number.
-        cur_page = update_page_number(cur_page, page_count, is_page_mod_key, btn,
+        curr_page = update_page_number(curr_page, prev_curr_page, num_pages, btn,
                                       input_text_page_num)
 
         # Get the current page and display it.
-        data, clip_pos = get_page(cur_page, page_display_list_cache, document,
+        data, clip_pos = get_page(curr_page, page_display_list_cache, document,
                                   window_size=window_size, zoom=zoom)
         image_element.Update(data=data)
 
